;Rev counter LED display

;Now called Rev3a
;13/02/10  Leading zero blanking being added



;	********* 04/02/10  LED segment mapping to PORTB ports changed to simplify stripboard layout ******

;Measure period of one revolution (Or one pulse if multiple pulses per revolution)

;15/01/10 being modified for 1 to 99 pulses per rev

;Use common anode display



;
;Changes from V2
;Interrupt routine changed to handle more than one interrupt while in the routine.
;

; Now wait until about 500 mS has elapsed since start of capture time. Keep track of number of CCP1 interrupts 
;since start of capture. Do calculation on first CCP1 interrupt that the accumulated count exceeds 500 mS worth of counts.
; As one count is 0.4 us then 500 mS = 500000/0.4 = 1250000  ( = 0x1312D0 )

; Use RA4 (Pin 3) to set pulses per rev.
; Hold down at power up to enter setup mode

; Need to be able to count to more than 255 CCP1 interupts to allow higher input frequency in multi pulse per rev version
; To do this two new variables are required CCP1_count_H and CCP1_count_copy_H
; CCP1_count will be renamed CCP1_count_L and CCP1_count_copy will be renamed CCP1_count_copy_L


; "S" Version  Mapping of PORTB bit to segments changed to simplify layout of stripboard version.

; NOTE segment drive is active low
;
;	RB0 (Pin 6)	Segment F
;	RB1 (Pin 7)	Segment C
;	RB2 (Pin 8)	Segment B
;	RB3 (Pin 9) Not used to drive Display This is the CCP input.
;	RB4 (Pin 10)	Segment A
;	RB5 (Pin 11)	Segment D
;	RB6 (Pin 12)	Segment E
;	RB7 (Pin 13)	Segment G


;Using 20 Mhz xtal system clock is 5 Mhz   (200ns)


	
;PIC16f628A, set for 20.00MHz XTAL, WDT OFF, POR on
;Instruction time at 20MHZ is 0.2 us




	list P=16f628
	#include "p16f628a.inc"
	__config _HS_OSC & _WDT_OFF & _PWRTE_ON & _LVP_OFF

;written in MASM

;				---- Constants ---
Timeout	EQU d'115' ;(gives about 3 seconds			;Maximum time allowed between input pulses 
;					Timeout *  26.2144 ms = maximum time allowed between input pulses
;					;Say 3 seconds This will allow a minimum speed of 20 RPM
;Update_Time	EQU d'50'	;Number of 5 ms between display updates (250 ms)
Update_Time	EQU d'100'	;Number of 5 ms between display updates (500 ms)


	CBLOCK	0x20	;GPR' starts at 0x20

Save_STATUS		;Save during interrupt
Save_W			;Save during interrupt
Save_PCLATH		;Save during interrupt
Save_FSR		;Save during interrupt	

Mux_time		;Time between digits
Timeout_Counter			;Used to test if maximum time allowed between input pulses has been exceeded

TMR1_Int_CountL
TMR1_Int_CountH

Captured_count0		;LSB
Captured_count1
Captured_count2
Captured_count3		;MSB


T0_Count0		;LSB
T0_Count1
T0_Count2
T0_Count3		;MSB


Accumulated_Count0		;LSB
Accumulated_Count1
Accumulated_Count2
Accumulated_Count3		;MSB

CCP1_count_L			;Number of CCP1 interrupts since start of capture (T0_time)
CCP1_count_H
CCP1_count_copy_L			;Value of CCP1_count when time reached to update display
CCP1_count_copy_H

Capture_State			;
;				0	Waiting for first CCP1 interrupt
;				1 	Waiting for next CCP1 interrupt
;

PPR_Val			;Value of pulses per revolution
			;Incremented in setup mode from 1 to 99 (Dec.) then back to 1
Increment_Timer		;Controls PPR value increment rate in setup mode

Save_PPR_Timer		;Used to time delay until PPR_Val is saved to EEPROM after button released


Dig0_Segments		;Segments to be displayed in digit 0 position  MSB
Dig1_Segments		;Segments to be displayed in digit 1 position
Dig2_Segments		;Segments to be displayed in digit 2 position
Dig3_Segments		;Segments to be displayed in digit 3 position

Dig_Count		;Current digit being displayed  (Only bits 0 & 1 used)

Flags
Flags2			;Used for testing to see when TMR1 and CCP1 interrupts occure at almost the same time
Flags3			;Bit 0 set when in setup mode

				;		rBin 4 Bytes (Binary input value)
rBin0				;		MSB
rBin1		
rBin2				

				;		rBCD 4 Bytes
rBCD				;		MSB
rBCD1			
rBCD2			
rBCD3				;		LSB

rDataCnt	
rBitCnt	
rCnt	

;				Registers used by maths routines
REGA0				;lsb
REGA1
REGA2
REGA3				;msb

REGB0				;lsb
REGB1
REGB2
REGB3				;msb

REGC0				;lsb
REGC1
REGC2
REGC3				;msb

MTEMP
MCOUNT

;		Registers used by EEPROM ROUTINES

DATA_EE_ADDR
DATA_EE_DATA


	ENDC			;Must end at 07F

LED_Port	EQU PORTB
Dig_Port	EQU PORTA


;					-----

;		Flag bit usage
;		Bit 0		Start of next digit time
;Flags
	#define Next_Dig_Flag Flags, 0	;Flag to indicate start of next digit time
	#define Display_Update_Flag Flags, 1	;Flag to indicate display update required
	#define CCP1_Flag Flags, 2
	#define Div_Err_Flag Flags, 3
	#define LZB_Flag Flags, 4		;Flag for leading zero blanking

;Flags3
	#define Setup_Flag Flags3,0
	#define	PPR_Change Flags3,1	;Button has been pressed so new PPR value needs saving to EEPROM


;Flags2
;	Bit 0	CCP1 interrupt
;	Bit 1	TMR1 interrupt
;	Bit 2	TMR0 interrupt


;==========================================================================
;
;I/O Pin useage

;	RA0		Output		Digit drive bit 4 LSB
;	RA1		Output		Digit drive bit 3
;	RA2		Output		Digit drive bit 2
;	RA3		Output		Digit drive bit 1 MSB	(Pin 2)
;	RA4/TOCK1	input		Push button to set counts per rev		(Pin 3)

;	RB3		Input	  Tacho input	         (Pin 9)  CCP1 Input from sensor

;	
;		NOTE THIS WAS FOR ORIGIN VERSION - Changed for stripboard version.
;				
;	RB0/INT		Output	  Segment A	         (Pin 6)   Ative low
;	RB1		Output	  Segment B	         (Pin 7)   Ative low
;	RB2		Output	  Segment C	         (Pin 8)   Ative low
;	RB4		Output    Segment D 		(Pin 10)   Ative low 
;	RB5		Output    Segment E		(Pin 11)   Ative low	
;	RB6		Output    Segment F 		(Pin 12)   Ative low	
;	RB7		Output    Segment G		(Pin 13)   Ative low
;

	#define	Button	PORTA,4

;					--------------------------

;			*** Define 7 Segment patterns   (Stripboard version) (NOTE active low.)

;			Assignment of PORTA bits to display segments
;			(These can be changed around to make track layout on PCB easier.)

;  Segment mapping for stripboard version
; NOTE segment drive is active low
;
;	RB0 (Pin 6)	Segment F
;	RB1 (Pin 7)	Segment C
;	RB2 (Pin 8)	Segment B
;	RB4 (Pin 10)	Segment A
;	RB5 (Pin 11)	Segment D
;	RB6 (Pin 12)	Segment E
;	RB7 (Pin 13)	Segment G

	#define	_7seg_0		b'10000000'
	#define	_7seg_1		b'11110001'
	#define	_7seg_2		b'00000011'
	#define	_7seg_3		b'01000001'
	#define	_7seg_4		b'01110000'
	#define	_7seg_5		b'01000100'
	#define	_7seg_6		b'00000100'
	#define	_7seg_7		b'11100001'
	#define	_7seg_8		b'00000000'
	#define	_7seg_9		b'01000000'
	#define	_7seg_A		b'00100000'
	#define	_7seg_B		b'00010100'
	#define	_7seg_C		b'10000110'
	#define	_7seg_D		b'00010001'
	#define	_7seg_E		b'00000110'
	#define	_7seg_F		b'00100110'

	#define _7seg_Dash	b'01111111'
	#define _7seg_Blank	0xFF		;Value to blank display (Note active low)


	#define	Seg_A		4
	#define	Seg_G		7
	

;			Assignment of PORTB bits to digit positions
;			(These can be changed around to make track layout on PCB easier.)
	#define Dig1	3
	#define Dig2	2
	#define Dig3	1
	#define Dig4	0

;					---------------------------


; define reset and interrupt vector start addresses

	org	0	  	; start at address 0000h
	goto	INIT		; normal service routines from Reset vector
	org     4		; interrupt vector 0004h, start interrupt routine here
	goto	Interupt

; **********************************************************************************************



;	Initialize 
;
        ORG 0x0005      	;Start of program memory


    	DE      "SV03a 13/02/10"
;
INIT    BCF INTCON,GIE  		;turn off global interrupts
        BTFSC INTCON,GIE 	;Confirm global interrupts off
        GOTO INIT

	MOVLW 	0x07        	; activate PORTA for PC16F628 as digital
	MOVWF	CMCON

	BSF 	STATUS,5	;Select register bank 1
        MOVLW	B'00001000'
        MOVWF	TRISB     	;Configure port B0-B2 and B4 - B7 as outputs ports B3 as input 
        MOVLW	B'00010000'
        MOVWF	TRISA     	;Port A0-A3 as output A4 as input
;

        MOVLW	B'10001000' 	; Weak pull-up on PORTB disabled (bit7=1) TMR0 prescaler assigned to watchdog  (Overflow every 51,2 us)
        MOVWF	OPTION_REG


;Configure TIMER1

	MOVLW	b'00000001'	;Enable timer1 interrupt
	MOVWF	PIE1

	BCF	STATUS,5	;Select register bank 0

	MOVLW	b'00010001'	;Configure timer 1
	MOVWF	T1CON		;Prescaler 1:2, Internal clock, Timer on  (Increment every 0.4 us)

;	Clear segment drives (Active low so set to 0xFF
	MOVLW	0xFF
	MOVWF	PORTB

;	Now test if setup button is pressed
;
	BTFSC	Button		;Low when pressed
	GOTO	Not_Setup


Wait_Button_Released
	BTFSS	Button		;Skip when released (High)
	GOTO	Wait_Button_Released

;	**** Now in setup mode ****

	BSF	Setup_Flag

;Set all segments off
	MOVLW	_7seg_Blank     ;(Active low)
	MOVWF	Dig0_Segments
	MOVWF	Dig1_Segments
	MOVWF	Dig2_Segments
	MOVWF	Dig3_Segments


;			Read the PPR value from EEPROM
	MOVLW	0x00		;EEPROM address to read
	CALL	EEREAD		;Value read returned in "W"
	MOVWF	PPR_Val		; Will be incremented by setup button

;turn on global interrupts

	MOVLW	B'11100000'		;Global interrupts, Peripheral interrupts, T0 interrupt enable
  	MOVWF 	INTCON  		;turn on required interrupts


	GOTO	Setup	;

Not_Setup
;Configure CCP1
	MOVLW	b'00000100'	;Capture mode, every falling edge
	MOVWF	CCP1CON

	BSF	STATUS, 5	;Select bank 1
	BSF	PIE1, CCP1IE	;Eneble CCP1 interrupt
	BCF	STATUS, 5	;Select bank 0


	MOVLW	0x0F
	MOVWF	Dig_Port	;Set all digit drives off (Active low)
	MOVLW	0xFF
	MOVWF	LED_Port	;Set value to 0xFF


	CLRF	Dig_Count	;Clear digit count (Note only bits 0 & 1 are used)
	CLRF	Capture_State

;Set initial display to read zero
	MOVLW	_7seg_Blank	      ;Blank top three digits
	MOVWF	Dig0_Segments
	MOVWF	Dig1_Segments
	MOVWF	Dig2_Segments
	MOVLW	_7seg_0			;Set bottom digit to read zero
	MOVWF	Dig3_Segments

;	BSF	PORTA, 4	;Set test LED off
	
	CLRF	TMR1L		;Clear timer 1
	CLRF	TMR1H

	BCF PIR1, 0x2  		;Clear CCP1IF

	BCF	Display_Update_Flag


;turn on global interrupts

	MOVLW	B'11100000'		;Global interrupts, Peripheral interrupts, T0 interrupt enable
  	MOVWF 	INTCON  		;turn on required interrupts

;			Read the PPR value from EEPROM
	MOVLW	0x00		;EEPROM address to read
	CALL	EEREAD		;Value read returned in "W"
	MOVWF	PPR_Val		;

;					---------  End if initialisation ---------------

	GOTO	MAIN

;*******************************************************************************

Interupt

;Save various registers

	MOVWF Save_W         	;Save W                            
	MOVF STATUS, W                         
	CLRF STATUS  		;Bank 0 will be selected
	MOVWF	Save_STATUS
	MOVF	PCLATH, W
	MOVWF	Save_PCLATH
	MOVF	FSR, W
	MOVWF	Save_FSR

	CLRF	Flags2


	BTFSS	INTCON, T0IF
	GOTO	Test_CCP1_Int	;Not Timer 0 interrupt

;				Timer0 used for display multiplexing and display update timming

Timer0_Int		;(Interrupts every 51.2 uS)   (Need digit time of about 5 mS) (This would need divide by about 97)



	DECFSZ	Mux_time,F	;
	GOTO	TMR0_End
Reload
	MOVLW	d'97' 		;Reload digit time counter for 1 digit diaplay time  
	MOVWF	Mux_time

Inc_Dig
	INCF	Dig_Count, F 		;Step to next digit position
	MOVLW	0x03
	ANDWF	Dig_Count, F	;Only use bits 0 and 1 as we only need to count 0 to 3
	BSF	Next_Dig_Flag

TMR0_End
	BCF INTCON, T0IF 	;Clear interrupt flag
;					-------------------


Test_CCP1_Int

	BTFSS	PIR1, CCP1IF
	GOTO	Test_Timer1_Int		;Not CCP1 interrupt
	BSF	Flags2, 0

	INCF	CCP1_count_L,F		;Increment count (Low byte) of CCP1 interrupts since T0 time
	BTFSC 	STATUS, Z 
	INCF	CCP1_count_H,F		;Increment count (High byte) of CCP1 interrupts since T0 time Z set


	MOVF	Capture_State, F	;Just to test if zero
	BTFSS	STATUS, Z
	GOTO	Not_T0
	INCF	Capture_State, F	;Increment to state 1

;				Copy count at last T0 time to T0_Count
	MOVF	Captured_count0, W	;LSB
	MOVWF	T0_Count0
	MOVF	Captured_count1, W
	MOVWF	T0_Count1	
	MOVF	Captured_count2, W
	MOVWF	T0_Count2
	MOVF	Captured_count3, W	;MSB
	MOVWF	T0_Count3


Not_T0
;		
;				Copy CCPR1L & H to Captured_count0 & 1
	MOVF	CCPR1L, W
	MOVWF	Captured_count0
	MOVF	CCPR1H, W
	MOVWF	Captured_count1

	MOVF	TMR1_Int_CountL, W
	MOVWF	Captured_count2
	MOVF	TMR1_Int_CountH, W
	MOVWF	Captured_count3


	BTFSS PIR1, 0        	;TMR1IF     (Has TMR1 interrupt also occured ?) 
	GOTO	CCP1_Int2
        BTFSC Captured_count1, 0x7		;Top bit of  CCPR1H      Capture occured after overflow  if clear
        GOTO CCP1_Int2		;  (The capture TMR1_Int_Count occured befor TMR1 interrupt)         
	INCF Captured_count2, F     		;Increment copy of TMR1_Int_Count  if bit 7 of CCPR1H NOT set                  
	BTFSC STATUS, Z                      
	INCF Captured_count3, F  

CCP1_Int2
;			------ Now calculate number of counts since T0 Time CCP interrupt ----


	MOVF	Captured_count0, W	
	MOVWF	Accumulated_Count0
	MOVF	Captured_count1, W	
	MOVWF	Accumulated_Count1
	MOVF	Captured_count2, W	
	MOVWF	Accumulated_Count2
	MOVF	Captured_count3, W	
	MOVWF	Accumulated_Count3


	MOVF	T0_Count0, W		;Subtract T0 count from Captured_count (Accumulated_Count)
	SUBWF	Accumulated_Count0, F
	MOVF	T0_Count1, W
	BTFSS	STATUS, C     	;Carry set for zero or positive
	INCFSZ	T0_Count1, W
	SUBWF	Accumulated_Count1, F
	MOVF	T0_Count2, W
	BTFSS	STATUS, C
	INCFSZ	T0_Count2, W
	SUBWF	Accumulated_Count2, F
	MOVF	T0_Count3, W
	BTFSS	STATUS, C
	INCFSZ	T0_Count3, W
	SUBWF	Accumulated_Count3, F
	BCF	STATUS, C

;								Note timer 1 increments every 0.4 us
;	Now test if accumulated count has exceeded 1250000  ( = 0x1312D0 ) (Counts in 500 mS)
; 250 ms = 625000   (= 98968 Hex)

;	May be better to do this in main loop

;				Test if	 Accumulated count     >= 0x1312D0  (Counts in 500 mS)
	MOVLW	0x00
	SUBWF	Accumulated_Count3, W
	BTFSS	STATUS, Z
	GOTO	Accumulated_Test_End
	MOVLW	0x13
	SUBWF	Accumulated_Count2, W
	BTFSS	STATUS, Z
	GOTO	Accumulated_Test_End
	MOVLW	0x12
	SUBWF	Accumulated_Count1, W
	BTFSS	STATUS, Z
	GOTO	Accumulated_Test_End
	MOVLW	0xD0
	SUBWF	Accumulated_Count0, W		;End of test 

Accumulated_Test_End

	BTFSS	STATUS, C	;Carry set if Accumulated_Count >= 0x001312D0  (Carry set if result zero or positive)
	GOTO	Not_time	;
	
;Set things ready for start of next timing period
	CLRF	Capture_State
;Save the number of pulses that occured in the timing period
	MOVF	CCP1_count_L, W
	MOVWF	CCP1_count_copy_L
	MOVF	CCP1_count_H, W
	MOVWF	CCP1_count_copy_H

;Set the pulse count back to zero
	CLRF	CCP1_count_L
	CLRF	CCP1_count_H



	BSF	Display_Update_Flag  ;New value to be displayed

Not_time
	MOVLW	Timeout			;Re load Timeout counter
	MOVWF	Timeout_Counter
                         
	BCF PIR1, 0x2  		;CCP1IF

;					--- End of CCP1 interrupt handler ---



;				Timer1 interupt handler 
;				Should come here every (1.6 us * 6553) =  104857.6uS= 104.8576 ms (This value not used now.)
Test_Timer1_Int
	BTFSS	PIR1,TMR1IF	;TMR1 interupt Interrupts every 0.4 * 256 * 256 uS = 26214.4 uS = 26.2144  mS
	GOTO	Int_End

	INCF	TMR1_Int_CountL,F
	BTFSC	STATUS, Z	;If overflow then increment Byte 3
	INCF	TMR1_Int_CountH,F

	MOVF	Timeout_Counter, F	; Just to test if it is zero (This is to deal with no input pulses)
	BTFSS	STATUS, Z		; This counter is reloaded every CCP1 interrupt
	DECF	Timeout_Counter, F
	
	BSF	Flags2, 1	
	BCF PIR1, TMR1IF 

Int_End				;Leave interrupt routine here


;Restore registers 

	MOVF	Save_FSR, W
	MOVWF	FSR
	MOVF	Save_PCLATH, W
	MOVWF	PCLATH	
	MOVF Save_STATUS, W                           
	MOVWF STATUS      ;Restore STATUS                     
	SWAPF Save_W, F                          
	SWAPF Save_W, W     ;Restore W    
	RETFIE
;			---------------------------------------------------------


MAIN				;Main program loop Just loop waiting for interupt.

	BTFSC	Next_Dig_Flag
	CALL	Display_Next_Dig

	BTFSC	Display_Update_Flag
	CALL	Update_Display		; Calculate and display current RPM value

;	Test for timeout   (No pulses for more than about three seconds)
	MOVF	Timeout_Counter, F	; Just to test if it is zero
	BTFSS	STATUS, Z
	GOTO	No_Clear

	;Force value displayed to zero as there have been no input pulses for timeout period

;	MOVLW	b'10000000'      ;7 segment pattern for 0 (Original segment mapping)
	MOVLW	_7seg_0
	MOVWF	Dig3_Segments

	MOVLW	_7seg_Blank		;Value to blank display
	MOVWF	Dig0_Segments
	MOVWF	Dig1_Segments
	MOVWF	Dig2_Segments

No_Clear
	GOTO	MAIN

;	------------------------------------------------------------------------------------------
Setup
	BCF	PPR_Change	;Clear the change flag initialy
Setup_Loop

	BTFSS	Next_Dig_Flag
	GOTO	Not_Next_Dig
	CALL	Display_Next_Dig
	DECFSZ	Increment_Timer,F	;Will overflow every 5 ms * 128 = 0.64 seconds
	GOTO	No_Tic
	MOVLW	0x80			;Number of 5 ms's between increments
	MOVWF	Increment_Timer
	DECF	Save_PPR_Timer,F

	BTFSC	Button		;Low when pressed
	GOTO	No_increment	;(Was No_Tic)
	BSF	PPR_Change	;Set to say value has changed and needs to be written to EEPROM
	MOVLW	D'10'
	MOVWF	Save_PPR_Timer	;Reset timeout value Used to save PPR value to EEPROM about 5 seconds after button released
	INCF	PPR_Val,F
	MOVF	PPR_Val,W
	SUBLW	d'99'
	BTFSC	STATUS,C
	GOTO	No_Tic
	MOVLW	0x01
	MOVWF	PPR_Val
	GOTO No_Tic

No_increment
	BTFSS	PPR_Change
	GOTO	No_Tic		;Button has not been pressed sinse last save.

	DECFSZ	Save_PPR_Timer,F	;When timer reaches zero save PPR_Val to EEPROM and clear PPR_Change bit
	GOTO	No_Tic
	BCF	PPR_Change
	MOVF	PPR_Val,W	;Put value to be saved in "W"
	BSF STATUS,RP0		;Select bank 1
	CLRF	EEADR		;Set EEPROM address to zero
	CALL	EWRITE		
	BCF STATUS,RP0		;Select bank 0


No_Tic
	CALL	PPR_Val_Disp	; Set Digx_Segments from PPR_Val

Not_Next_Dig

	GOTO	Setup_Loop



;	***************************************************************************************************


	
;				+++++++++++++++ Subroutines from here +++++++++++++++++++++++++



;-----------------------------------------------------------------------------
;
; Segment patterns
;
;-----------------------------------------------------------------------------
;  These values are for common anode displays

		ORG	0x100		;To avoid crossing a boundry in this table.

HEX2SEGMENTS    
		CLRF	PCLATH
		INCF	PCLATH, F	;Set PCLATH to 0x01 to point to this page

		ANDLW   b'00001111'      ;Use only lower nibble
                ADDWF   PCL,F           ;Add offset to current PC

;		*** These bit patterns could be changed to assign segments to different I/O pins to make board layout easier ***


;These bit patterns are for the following segment to bit assignments

;These are the origin  assignments NOW DIFFERENT for stripboard version
;	A	RB0
;	B	RB1
;	C	RB2
;	D	RB4
;	E	RB5
;	F	RB6
;	G	RB7

SEGMENTS        
; 		 RETLW   b'10000000'      ;0	NOTE low to illuminate segment
;                RETLW   b'11110001'      ;1
;                RETLW   b'01000100'      ;2
;                RETLW   b'01100000'      ;3
;                RETLW   b'00110001'      ;4
;                RETLW   b'00100010'      ;5
;                RETLW   b'00000010'      ;6
;                RETLW   b'11110000'      ;7
;                RETLW   b'00000000'      ;8
;                RETLW   b'00100000'      ;9
;                RETLW   b'00010000'      ;A
;                RETLW   b'00000011'      ;B
;                RETLW   b'10000110'      ;C
;                RETLW   b'01000001'      ;D
;                RETLW   b'00000110'      ;E
;                RETLW   b'00010110'      ;F

;					-----------------------------------

		RETLW	_7seg_0
		RETLW	_7seg_1
		RETLW	_7seg_2
		RETLW	_7seg_3
		RETLW	_7seg_4
		RETLW	_7seg_5
		RETLW	_7seg_6
		RETLW	_7seg_7	
		RETLW	_7seg_8
		RETLW	_7seg_9
		RETLW	_7seg_8
		RETLW	_7seg_9
		RETLW	_7seg_A		;Note A to F are not used
		RETLW	_7seg_B
		RETLW	_7seg_C
		RETLW	_7seg_D
		RETLW	_7seg_E
		RETLW	_7seg_F






;					-----------------------------------

 
Update_Display

	BCF	INTCON, GIE	;Disable all interrupts during copy

	MOVF	Accumulated_Count0, W	;LSB
	MOVWF	REGA0
	MOVF	Accumulated_Count1, W
	MOVWF	REGA1	
	MOVF	Accumulated_Count2, W
	MOVWF	REGA2
	MOVF	Accumulated_Count3, W	;MSB
	MOVWF	REGA3

	MOVF	CCP1_count_copy_L, W
	MOVWF	REGB0
	MOVF	CCP1_count_copy_H, W
	MOVWF	REGB1

	BSF	INTCON, GIE	;Re enable interrupts

;	CLRF	REGB1
	CLRF	REGB2
	CLRF	REGB3


;				Divide accumulated counts by number of CCP1 interrupts

	CALL	divide		;Result will be in REGA

	MOVF	REGA0, W	;LSB
	MOVWF	REGB0
	MOVF	REGA1, W
	MOVWF	REGB1	
	MOVF	REGA2, W
	MOVWF	REGB2
	MOVF	REGA3, W	;MSB
	MOVWF	REGB3

;		Load value into dividend   150000000 in HEX is 8F0D180 (This value fo 1 pulse per rev.)


	MOVLW	0x08
	MOVWF	REGA3

	MOVLW	0xF0
	MOVWF	REGA2

	MOVLW	0xD1
	MOVWF	REGA1

	MOVLW	0x80
	MOVWF	REGA0

	CALL	divide		;Divide number of counts (0.4 uS) for 1 rev (1 CCP1 interrupt) into 150000000

;	********************************************************************************************************
;If the spindle generated more than one pulse per rev we would need to divide this result by the number of pulses per rev here

;	Dividend will already be in REGA0 - REGA3
	CLRF	REGB1
	CLRF	REGB2
	CLRF	REGB3
	MOVF	PPR_Val,W
	MOVWF	REGB0
	CALL	divide






;	********************************************************************************************************

;			Now need to test that reading is not greater than 9999 decimal = 0x270F


	MOVF	REGA3, W	;These two must be zero
	IORWF	REGA2, W
	BTFSS	STATUS, Z
	GOTO 	Oflow

	MOVF	REGA1, W
	SUBLW	0x26
	BTFSC	STATUS, C 	;Carry set for zero or positive
	GOTO 	No_Oflow

	MOVF	REGA0, W
	SUBLW	0x0F
	BTFSS	STATUS, C 	;Carry set for zero or positive
	GOTO 	Oflow

No_Oflow
	MOVF	REGA2, W
	MOVWF	rBin0		;MSB
	MOVF	REGA1, W
	MOVWF	rBin1
	MOVF	REGA0, W
	MOVWF	rBin2		;LSB

	CALL	BintoBCD


;		Code being added here for leading zero blanking

;							Most significant digit (Digit 0)

	BSF	LZB_Flag		;Set leading zero blanking flag to start
	SWAPF	rBCD2, W		;Value will be in lower nibble of "W"
	ANDLW	b'00001111'		;Testing for value of zero
	BTFSS	STATUS, Z
	GOTO	LZB01
	MOVLW	_7seg_Blank			;Blank all segments (Note active low)
	GOTO	LZB02
LZB01
	BCF	LZB_Flag		;Clear flag	
	CALL 	HEX2SEGMENTS
LZB02
	MOVWF	Dig0_Segments		;Most significant

;					Next digit	(Digit 1)

	MOVF	rBCD2, W		;Value will be in lower nibble of "W"
	BTFSS	LZB_Flag
	GOTO	LZB03			;Top digit not zero
	ANDLW	b'00001111'		;Testing for value of zero
	BTFSS	STATUS, Z
	GOTO	LZB03
	MOVLW	_7seg_Blank			;Blank all segments (Note active low)
	GOTO	LZB04

LZB03
	BCF	LZB_Flag		;Clear flag
	CALL	HEX2SEGMENTS
LZB04
	MOVWF	Dig1_Segments

;					Next digit (Digit 2)

	SWAPF	rBCD3, W		;Value will be in lower nibble of "W"
	BTFSS	LZB_Flag
	GOTO	LZB05			;Previous digits not zero
	ANDLW	b'00001111'		;Testing for value of zero
	BTFSS	STATUS, Z
	GOTO	LZB05
	MOVLW	_7seg_Blank			;Blank all segments (Note active low)
	GOTO	LZB06

LZB05
	CALL	HEX2SEGMENTS
LZB06
	MOVWF	Dig2_Segments






;					Next digit (Digit 3)

	MOVF	rBCD3, W		;Value will be in lower nibble of "W"	
	CALL	HEX2SEGMENTS
	MOVWF	Dig3_Segments

	BCF	Display_Update_Flag
	RETURN

Oflow
	MOVLW	_7seg_Dash			;b'01111111'	
	MOVWF	Dig0_Segments
	MOVWF	Dig1_Segments
	MOVWF	Dig2_Segments
	MOVWF	Dig3_Segments

	BCF	Display_Update_Flag
	RETURN

;	-----------------------------------------------------------------------------------------
;
;			Update PPR display value in setup mode
PPR_Val_Disp	; Set Digx_Segments from PPR_Val

	CLRF	rBin0		;MSB
	CLRF	rBin1
	MOVF	PPR_Val, W
	MOVWF	rBin2		;LSB

	CALL	BintoBCD

	MOVLW	_7seg_Blank
	MOVWF	Dig0_Segments	;blank top two digits (Active low)
	MOVWF	Dig1_Segments

	BTFSC	PPR_Change
	BCF	Dig0_Segments, Seg_A       ;Light a segment if change bit set This is for stripboard version.

	SWAPF	rBCD3, W
	CALL	HEX2SEGMENTS
	MOVWF	Dig2_Segments

	MOVF	rBCD3, W
	
	CALL	HEX2SEGMENTS
	MOVWF	Dig3_Segments

;	BCF	Display_Update_Flag

	RETURN

;*****************************************************************************
;
;	Function : Display next digit		(4 Digit multiplex display driver)
;
;*****************************************************************************

;	**** NOTE Dig_Port bit assigment to digit positions could be chaned to make board layout easier *****

;
Display_Next_Dig

	MOVLW	0x0F
	IORWF	Dig_Port,F		;All digit drives off (Active low) Bits 0 - 3

	BTFSC	Dig_Count, 1
	GOTO	TwoorThree
ZeroorOne
	BTFSC	Dig_Count, 0
	GOTO	One
Zero
	MOVF	Dig0_Segments, W
	MOVWF	LED_Port		;Output it.
	BCF	Dig_Port,Dig1			;Clear bit  (Enable digit position 1) (Most significant
	GOTO	Disp_End
One
	MOVF	Dig1_Segments, W
	MOVWF	LED_Port		;Output it to LED_Port
	BCF	Dig_Port,Dig2			;Clear bit  (Enable digit position 2)
	GOTO	Disp_End

TwoorThree
	BTFSC	Dig_Count, 0
	GOTO	Three
Two
	MOVF	Dig2_Segments, W
	MOVWF	LED_Port		;Output it.
	BCF	Dig_Port,Dig3			;Clear bit  (Enable digit position 3)
	GOTO	Disp_End
Three
	MOVF	Dig3_Segments, W
	MOVWF	LED_Port		;Output it.
	BCF	Dig_Port,Dig4			;Clear bit  (Enable digit position 4)

Disp_End
	BCF	Next_Dig_Flag		;Clear flag to say digit is being displayed
	RETURN


;			--------------------------------------------------------------

;Input	in rBin0 to rBin+2   (3 bytes)
;Result in rBCD to rBCD+3   (4 Bytes - two BCD characters per byte)
		

BintoBCD	
			BCF	STATUS,C
			MOVLW	D'24'		; 24-Bits
			MOVWF	rBitCnt		; Cycle counter
			CLRF	rBCD		; Clear result area
			CLRF	rBCD+1		;
			CLRF	rBCD+2		;
			CLRF	rBCD+3		;

Loop			MOVLW	rBCD		; (Most significant byte)
			MOVWF	FSR
			MOVLW	4
			MOVWF	rCnt		;

BintoBCD1		
			MOVLW	0x33
			ADDWF	INDF,F		;
			BTFSC	INDF,3
			ANDLW	0xF0
			BTFSC	INDF,7
			ANDLW	0x0F
			SUBWF	INDF,F
			INCF	FSR,F
			DECFSZ	rCnt,F
			GOTO	BintoBCD1	;Loop (4 times round loop)

			RLF	rBin0+2,F	; Get another bit
			RLF	rBin0+1,F	;
			RLF	rBin0+0,F	;
			RLF	rBCD+3,F	; Put it into BCD
			RLF	rBCD+2,F	;
			RLF	rBCD+1,F	;
			RLF	rBCD+0,F	;
			DECFSZ	rBitCnt,F	; All done?
			GOTO	Loop		; No, loop
			RETURN

;						----------------------------------



;				**** Maths subroutines ****

;	*********************************************************************************************************
;	*													*
;	*				32 Bit divide								*
;	*													*
;	*	Dividend in REGA0, REGA1, REGA2, REGA3	(REGA3 most significant.)				*
;	*	Divisor in REGB0, REGB1, REGB2, REGB3	(REGB3 most significant.)				*
;	*	Result in REGA0, REGA1, REGA2, REGA3	(REGA3 most significant.)				*
;	*	Remaindor in REGC0, REGC1, REGC2, REGC3	(REGC3 most significant.)				*
;	*	Carry set on return if overflow or division by zero						*
;	*													*
;	*********************************************************************************************************
;*** 32 BIT DIVIDE ***
;REGA / REGB -> REGA
;Remainder in REGC
;Return carry set if overflow or division by zero

divide
	MOVF	REGB0, W		;Check for division by zero
	IORWF	REGB1, W
	IORWF	REGB2, W
	IORWF	REGB3, W
	SUBLW	0
	BTFSC	STATUS, C	;Carry set if result is zero or positive
	GOTO	Divide_Exit	;return			;Overflow  (IE divide by zero) (Carry will be set)


	CLRF	REGC0		;Clear remainder
	CLRF	REGC1
	CLRF	REGC2
	CLRF	REGC3

	MOVLW	D'32'		;Loop counter
	MOVWF	MCOUNT

divloop	
	RLF	REGA0,f		;LSB	;Shift dividend (REGA) msb into remainder (REGC)
	RLF	REGA1,f
	RLF	REGA2,f
	RLF	REGA3,f		;MSB  (Top bit goes to carry)

	RLF	REGC0,f		;Carry goes to bottom bit
	RLF	REGC1,f
	RLF	REGC2,f
	RLF	REGC3,f

	MOVF	REGB3, W	;Test if remainder (REGC) >= divisor (REGB)
	SUBWF	REGC3, W
	BTFSS	STATUS, Z
	GOTO	Div_test_end
	MOVF	REGB2, W
	SUBWF	REGC2, W
	BTFSS	STATUS, Z
	GOTO	Div_test_end
	MOVF	REGB1, W
	SUBWF	REGC1, W
	BTFSS	STATUS, Z
	GOTO	Div_test_end
	MOVF	REGB0, W
	SUBWF	REGC0, W		;End of test 
Div_test_end	
	BTFSS	STATUS, C	;Carry set if remainder >= divisor  (Carry set if result zero or positive)
	GOTO	No_Go		;if remainder not >= divisor (If less than divisor)

	MOVF	REGB0, W		;Subtract divisor (REGB) from remainder (REGC)
	SUBWF	REGC0, F
	MOVF	REGB1, W
	BTFSS	STATUS, C
	INCFSZ	REGB1, W		;
	SUBWF	REGC1, F
	MOVF	REGB2, W
	BTFSS	STATUS, C
	INCFSZ	REGB2, W
	SUBWF	REGC2, F
	MOVF	REGB3, W
	BTFSS	STATUS, C
	INCFSZ	REGB3, W
	SUBWF	REGC3, F		;End of subtraction  Remainder in REGC
	BCF	STATUS, C	;Clear carry
	BSF	REGA0, 0		;Set quotient bit

No_Go	DECFSZ	MCOUNT, F	;Next
	GOTO	divloop		;Loop

	GOTO	Divide_Exit


Divide_Exit
	RETURN

;			--- End of divide ---

;				**** End of maths routines ****
  
;*****************************************************************************        
;
; subroutine to read from EEPROM

;
;   Function : EEREAD    		       
;		
;   Input:     EEPROM address to read from  (In "W" register)
;
;   Output:    Date read from EEPROM  (In "W" register)
;
;*****************************************************************************  
; subroutine to read EEPROM memory  (For PIC16F628A)
EEREAD
	bsf 	STATUS,RP0	; select memory bank 1
	movwf 	EEADR		; write EEADR
	bsf	EECON1,RD	; read EEPROM	
	movf	EEDATA,W		; EEPROM value in W
	BCF	STATUS,RP0	;Select bank 0

	return
      
;*****************************************************************************
;
; subroutine to write to EEPROM
;
;   Function : EWRITE
;     		Writes data in "W" register to internal EEPROM
;		Address of EEPROM to be written must first be put in EEADR register        
;		
;   Input:     Value to write to internal EEPROM
;
;*****************************************************************************       

; subroutine to write to EEPROM     (For PIC16F628)

EWRITE

	BSF STATUS,RP0		;Select bank 1
	BSF EECON1,WREN 	;
	MOVWF	EEDATA
EWR01
	BCF INTCON,GIE 		;Disable INTs.
	BTFSC INTCON,GIE
	GOTO EWR01 		;Wait to complete

	MOVLW 0x55 		;
	MOVWF EECON2 		;Write 55h
	MOVLW 0xAA 		;
	MOVWF EECON2 		;Write AAh

	BSF EECON1,WR 		;Set WR bit to  begin write
	BCF	STATUS,RP0	;Select bank 0
	BSF INTCON,GIE 		;Enable INTs.
	RETURN



;			---------------------------------
;Write starting values to EEPROM memory (Only location zero used)

         ORG 0x2100
	DE	0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00


        END            	;final line. This line must be retained.


